// ///////////////////  ***********************  \\\\\\\\\\\\\\\\\\\\ //
//
//  SoGlutRenderArea, version 2.0
//
//  a simple window component like SoXtRenderArea for Inventor
//
//  Copyright (C) 2001-2002 by PC John. All rights reserved.
//
//  This game is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public License
//  version 2.1 as published by the Free Software Foundation. See the
//  file LICENSE.LGPL at the root directory of the distribution for
//  more details.
//
//  If you want to use Sector 5 code for applications not compatible 
//  with the LGPL license, please contact PC John at pcjohn@email.cz.
//
// \\\\\\\\\\\\\\\\\\\  ***********************  //////////////////// //


#ifndef SO_GLUT_RENDER_AREA_H_
#define SO_GLUT_RENDER_AREA_H_


#define SoXt SoGlut
#define SoXtRenderArea SoGlutRenderArea

#include <Inventor/SoSceneManager.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/fields/SoSFTime.h>




class SoGlut;
class SoGlutRenderArea;


typedef union Widget {
  int asInt;
  void *asPointer;
  Widget(int i)  { asInt = i; }
  Widget(void *p)  { asPointer = p; }
  operator int()  { return asInt; }
  operator void*()  { return asPointer; }
} Widget;



class SoGlut
{
friend class SoGlutRenderArea;
public:

  static Widget init(const char *appName, const char *className = "Inventor");
  static Widget init(int argc, char ** argv);
  static void done();
  static void mainLoop();

  static void show(Widget widget = 0);
  static void hide(Widget widget = 0);

private:

  static SoGlutRenderArea *windowsList;
  static bool widgetVisible;

  static void addWindow(SoGlutRenderArea *window);
  static void removeWindow(SoGlutRenderArea *window);

  static SoGlutRenderArea* getWindowFromHandle(Widget id);

};



class SoGlutRenderArea
{
friend class SoGlut;

public:

  SoGlutRenderArea(Widget parent = NULL,
    const char *name = NULL,
    SbBool buildInsideParent = TRUE,
    SbBool getMouseInput = TRUE,
    SbBool getKeyboardInput = TRUE);
  ~SoGlutRenderArea();

  // SoGlut extensions
  void setPosition(SbVec2s pos);
  SbVec2s getPosition() const;
  void centerWindow();

  void setFullScreenParams(int width, int height, int bpp = 0)  { fsWidth=width; fsHeight=height; fsBPP = bpp; }
  void getResolutionParams(int *width, int *height, int *bpp) const { if (width) *width = fsWidth; if (height) *height = fsHeight; if (bpp) *bpp = fsBPP; }
  void setFullScreen(bool b);
  bool isFullScreen() const  { return fullScreen; }

  typedef struct { int width; int height; int bpp; } Resolution;
  static int getResolutionList(const Resolution *&resList, int depthBits = 0);
  static bool isResolutionSupported(int width, int height, int bpp = 0);
  bool isResolutionSupported(int bpp = 0) const { return isResolutionSupported(getSize()[0], getSize()[1], bpp); }

  // unsupported SoGLWidget functions - glut windows have always border,
  // functions leaved empty for API compatibility
  void setBorder(SbBool onOrOff)  {};
  SbBool isBorder() const  { return true; }
  //void setWindowCloseCallback(SoXtComponentCB *func, void *data = NULL);
  
  // unsupported from SoXtRenderArea
  //void setEventCallback(SoXtRenderAreaEventCB *fcn, void *userData = NULL)

  // from SoXtRenderArea
  virtual void setSceneGraph(SoNode *newScene)  { sceneManager->setSceneGraph(newScene); };
  virtual SoNode* getSceneGraph()  { return sceneManager->getSceneGraph(); };

  void setBackgroundColor(const SbColor &c)  { sceneManager->setBackgroundColor(c); };
  const SbColor& getBackgroundColor() const  { sceneManager->getBackgroundColor(); };

  void setViewportRegion(const SbViewportRegion &newRegion);
  const SbViewportRegion& getViewportRegion() const;

  void setTransparencyType(SoGLRenderAction::TransparencyType type);
  SoGLRenderAction::TransparencyType getTransparencyType() const;

  void setAntialiasing(SbBool smoothing, int numPasses)  { sceneManager->setAntialiasing(smoothing, numPasses); };
  void getAntialiasing(SbBool &smoothing, int &numPasses) const  { sceneManager->getAntialiasing(smoothing, numPasses); };

  void setClearBeforeRender(SbBool trueOrFalse, SbBool zbTrueOrFalse=TRUE)
    { clearColorBuffer = trueOrFalse; clearZBuffer = zbTrueOrFalse; };
  SbBool isClearBeforeRender() const  { return clearColorBuffer; };
  SbBool isClearZBufferBeforeRender() const  { return clearZBuffer; };

  void setAutoRedraw(SbBool trueOrFalse)  { trueOrFalse ? autoRedraw = true : autoRedraw = false; };
  SbBool isAutoRedraw() const  { return autoRedraw; };
  void setRedrawPriority(unsigned long priority)  { sceneManager->setRedrawPriority(priority); };
  unsigned long getRedrawPriority() const  { return sceneManager->getRedrawPriority(); };
  static unsigned long getDefaultRedrawPriority()  { return SoSceneManager::getDefaultRedrawPriority(); };

  void render()  { sceneManager->render(clearColorBuffer, clearZBuffer); };
  void scheduleRedraw()  { sceneManager->scheduleRedraw(); };

  void setGLRenderAction(SoGLRenderAction *ra)  { sceneManager->setGLRenderAction(ra); };
  SoGLRenderAction* getGLRenderAction() const  { return sceneManager->getGLRenderAction(); };

  void setSceneManager(SoSceneManager *sm);
  SoSceneManager* getSceneManager() const  { return sceneManager; };

  // from SoXtGLWidget
  void setDoubleBuffer(SbBool onOrOff);
  SbBool isDoubleBuffer();

  // from SoXtComponent
  virtual void show();
  virtual void hide();
  SbBool isVisible()  { return visible && SoGlut::widgetVisible; };

  void setSize(const SbVec2s &size);
  SbVec2s getSize() const;

  void setTitle(const char *newTitle);
  const char* getTitle() const  { return title; };

  void setIconTitle(const char *newIconTitle);
  const char* getIconTitle() const  { return iconTitle; }

  Widget getWidget() const  { return handle; };
  static SoGlutRenderArea* getComponent(Widget w)  { return SoGlut::getWindowFromHandle(w); }

public:

  // internal functions
  void redraw_cb();
  void reshape_cb(int w, int h);
  void keyboard_cb(int key, bool down, bool spec, int x, int y);
  void mouse_motion_cb(int x, int y);
  void mouse_button_cb(SoMouseButtonEvent::Button b, bool down, int x, int y);
  void makeCurrent();

protected:

  int handle;

  SoGlutRenderArea *nextWindowsListItem;

  SoSceneManager *sceneManager;
  bool ownedSceneManager;

  bool visible;
  SbVec2s winSize;
  bool fullScreen;
  int fsWidth, fsHeight;
  int fsBPP;
  char *title;                           
  char *iconTitle;
  unsigned int mode;
  SbVec2s position;
  SbBool clearColorBuffer;
  SbBool clearZBuffer;
  bool autoRedraw;
  SbBool getMouseInput;
  SbBool getKeyboardInput;

  virtual void doShow();
  virtual void doHide();

  void makeEvent(SoEvent *event, int x, int y, int modifiers);

  void createWindow();
  void destroyWindow();
  void recreateWindow();
  void sizeChanged(int w, int h);

  bool doEnterFullScreen();
  void doLeaveFullScreen();

private:

  SoKeyboardEvent keyEvent;
  SoLocation2Event mouseMotionEvent;
  SoMouseButtonEvent mouseButtonEvent;

};

#endif /* _SO_GLUT_RENDER_AREA_ */





